@@ -10,10 +10,10 @@ from logger import log
 from store import Project, ModuleNotFound, ModuleNeedsAnalysis, \
      ModuleSaveError, get_pythoscope_path, get_points_of_entry_path, \
      get_code_trees_path
-from util import samefile
+from compat import samefile
 
 
-__version__ = '0.4.1'
+__version__ = '0.4.2'
 
 BUGTRACKER_URL = "https://bugs.launchpad.net/pythoscope"
 USAGE = """Pythoscope usage:
@@ -95,7 +95,7 @@ def find_project_directory(path):
     else:
         return find_project_directory(os.path.join(path, os.path.pardir))
 
-def init_project(path):
+def init_project(path, skip_inspection=False):
     pythoscope_path = get_pythoscope_path(path)
 
     try:
@@ -106,9 +106,10 @@ def init_project(path):
     except OSError, err:
         fail("Couldn't initialize Pythoscope directory: %s." % err.strerror)
 
-    log.debug("Performing initial static inspection of the project source code.")
     project = Project.from_directory(path)
-    inspect_project_statically(project)
+    if not skip_inspection:
+        log.debug("Performing initial static inspection of the project source code.")
+        inspect_project_statically(project)
     project.save()
 
 def generate_tests(modules, force, template):
@@ -190,5 +191,5 @@ def main():
     except KeyboardInterrupt:
         log.info("Interrupted by the user.")
     except Exception: # SystemExit gets through
-        log.error("Oops, it seems internal Pythoscope error occured. Please file a bug report at %s\n" % BUGTRACKER_URL)
+        log.error("Oops, it seems that an internal Pythoscope error occurred. Please file a bug report at %s\n" % BUGTRACKER_URL)
         raise
