@@ -1,14 +1,16 @@
-from pythoscope.astvisitor import descend, parse_fragment, ASTVisitor, \
-    EmptyCode
+from pythoscope.astvisitor import descend, ASTVisitor
+from pythoscope.astbuilder import parse_fragment, EmptyCode
 from pythoscope.logger import log
 from pythoscope.generator.adder import add_test_case_to_project
-from pythoscope.serializer import CompositeObject, ImmutableObject, MapObject, \
-    UnknownObject, SequenceObject, SerializedObject, can_be_constructed
+from pythoscope.serializer import BuiltinException, CompositeObject, \
+    ImmutableObject, MapObject, UnknownObject, SequenceObject, \
+    SerializedObject, can_be_constructed, is_serialized_string
 from pythoscope.store import Class, Function, FunctionCall, TestClass, \
     TestMethod, ModuleNotFound, UserObject, MethodCall, Method, Project, \
     GeneratorObject
+from pythoscope.compat import any, set, sorted
 from pythoscope.util import camelize, compact, counted, flatten, \
-    key_for_value, pluralize, set, sorted, underscore, union
+    key_for_value, pluralize, underscore, union
 
 
 # :: [string] -> string
@@ -27,6 +29,12 @@ def type_as_string(object):
 
     return object.type_name
 
+# :: string -> string
+def todo_value(value):
+    """Wrap given value in a <TODO: value> block.
+    """
+    return "<TODO: %s>" % value
+
 class CallString(str):
     """A string that holds information on the function/method call it
     represents.
@@ -53,8 +61,8 @@ def constructor_as_string(object, assigned_names={}):
     """For a given object (either a SerializedObject or a list of them) return
     a string representing a code that will construct it.
 
-    >>> from test.helper import EmptyProjectExecution
-    >>> serialize = EmptyProjectExecution().serialize
+    >>> from test.helper import make_fresh_serialize
+    >>> serialize = make_fresh_serialize()
 
     It handles built-in types
         >>> constructor_as_string(serialize(123))
@@ -106,45 +114,60 @@ def constructor_as_string(object, assigned_names={}):
         init_call = object.get_init_call()
         if init_call:
             args = init_call.input
-        return call_as_string(object.klass.name, args)
+        return call_as_string(object.klass.name, args,
+                              init_call and init_call.definition)
     elif isinstance(object, ImmutableObject):
         return CallString(object.reconstructor, imports=object.imports)
     elif isinstance(object, CompositeObject):
         try:
             reconstructors, imports, uncomplete = zip(*get_contained_objects_info(object, assigned_names))
-        except ValueError:
+        # In Python <= 2.3 zip can raise TypeError if no arguments were provided.
+        # All Pythons can raise ValueError because of the wrong unpacking.
+        except (ValueError, TypeError):
             reconstructors, imports, uncomplete = [], [], []
         return CallString(object.constructor_format % ', '.join(reconstructors),
                           imports=union(object.imports, *imports),
                           uncomplete=any(uncomplete))
     elif isinstance(object, UnknownObject):
-        return CallString("<TODO: %s>" % object.partial_reconstructor, uncomplete=True)
+        return CallString(todo_value(object.partial_reconstructor), uncomplete=True)
     else:
         raise TypeError("constructor_as_string expected SerializedObject at input, not %s" % object)
 
+# :: ([SerializedObject], {SerializedObject: str}) -> [(str, set, bool)]
+def get_objects_collection_info(objs, assigned_names):
+    for obj in objs:
+        cs = constructor_as_string(obj, assigned_names)
+        yield (cs, cs.imports, cs.uncomplete)
+
+# :: ({SerializedObject: SerializedObject}, {SerializedObject: str}) -> [(str, set, bool)]
+def get_objects_mapping_info(mapping, assigned_names):
+    for key, value in mapping:
+        keycs = constructor_as_string(key, assigned_names)
+        valuecs = constructor_as_string(value, assigned_names)
+        yield ("%s: %s" % (keycs, valuecs),
+               union(keycs.imports, valuecs.imports),
+               keycs.uncomplete or valuecs.uncomplete)
+
 # :: (CompositeObject, {SerializedObject: str}) -> [(str, set, bool)]
 def get_contained_objects_info(obj, assigned_names):
     """Return a list of tuples (reconstructor, imports, uncomplete) describing
     each object contained within a composite object.
     """
     if isinstance(obj, SequenceObject):
-        for subobj in obj.contained_objects:
-            cs = constructor_as_string(subobj, assigned_names)
-            yield(cs, cs.imports, cs.uncomplete)
+        return get_objects_collection_info(obj.contained_objects, assigned_names)
     elif isinstance(obj, MapObject):
-        for key, value in obj.mapping:
-            keycs = constructor_as_string(key, assigned_names)
-            valuecs = constructor_as_string(value, assigned_names)
-            yield("%s: %s" % (keycs, valuecs),
-                  union(keycs.imports, valuecs.imports),
-                  keycs.uncomplete or valuecs.uncomplete)
+        return get_objects_mapping_info(obj.mapping, assigned_names)
+    elif isinstance(obj, BuiltinException):
+        return get_objects_collection_info(obj.args, assigned_names)
+    else:
+        raise TypeError("Wrong argument to get_contained_objects_info: %r." % obj)
 
-# :: (string, dict, {SerializedObject: str}) -> CallString
-def call_as_string(object_name, args, assigned_names={}):
+# :: (string, dict, Definition, {SerializedObject: str}) -> CallString
+def call_as_string(object_name, args, definition=None, assigned_names={}):
     """Generate code for calling an object with given arguments.
 
-    >>> from test.helper import EmptyProjectExecution
-    >>> serialize = EmptyProjectExecution().serialize
+    >>> from test.helper import make_fresh_serialize
+    >>> serialize = make_fresh_serialize()
 
     >>> call_as_string('fun', {'a': serialize(1), 'b': serialize(2)})
     'fun(a=1, b=2)'
@@ -169,20 +192,79 @@ def call_as_string(object_name, args, assigned_names={}):
     Uses names already assigned to objects instead of inlining their
     construction code.
         >>> mutable = serialize([])
-        >>> call_as_string('merge', {'seq1': mutable, 'seq2': serialize([1,2,3])}, {mutable: 'alist'})
+        >>> call_as_string('merge', {'seq1': mutable, 'seq2': serialize([1,2,3])},
+        ...     None, {mutable: 'alist'})
         'merge(seq1=alist, seq2=[1, 2, 3])'
+
+    Puts varargs at the end of arguments list.
+        >>> call_as_string('build_url',
+        ...     {'proto': serialize('http'), 'params': serialize(('user', 'session', 'new'))},
+        ...     Function('build_url', ['proto', '*params']))
+        "build_url(proto='http', 'user', 'session', 'new')"
+
+    Works for lone varargs too.
+        >>> call_as_string('concat', {'args': serialize(([1,2,3], [4,5], [6]))},
+        ...     Function('concat', ['*args']))
+        'concat([1, 2, 3], [4, 5], [6])'
+
+    Uses assigned name for varargs as well.
+        >>> args = serialize((1, 2, 3))
+        >>> call_as_string('add', {'args': args}, Function('add', ['*args']), {args: 'atuple'})
+        'add(*atuple)'
+
+    Inlines extra keyword arguments in the call...
+        >>> call_as_string('dict', {'kwargs': serialize({'one': 1, 'two': 2})},
+        ...     Function('dict', ['**kwargs']))
+        'dict(one=1, two=2)'
+
+    ...even when they are combined with varargs.
+        >>> call_as_string('wrap', {'a': serialize((1, 2, 3)), 'k': serialize({'x': 4, 'y': 5})},
+        ...     Function('wrap', ['*a', '**k']))
+        'wrap(1, 2, 3, x=4, y=5)'
+
+    Uses assigned name for kwarg if present.
+        >>> kwargs = serialize({'id': 42, 'model': 'user'})
+        >>> call_as_string('filter_params', {'kwargs': kwargs},
+        ...    Function('filter_params', ['**kwargs']), {kwargs: 'params'})
+        'filter_params(**params)'
     """
     arguments = []
+    varargs = []
+    kwargs = []
     uncomplete = False
     imports = set()
     for arg, value in sorted(args.iteritems()):
-        constructor = constructor_as_string(value, assigned_names)
-        uncomplete = uncomplete or constructor.uncomplete
-        imports.update(constructor.imports)
-        arguments.append("%s=%s" % (arg, constructor))
-    return CallString("%s(%s)" % (object_name, ', '.join(arguments)),
+        if definition and definition.is_vararg(arg):
+            if value in assigned_names.keys():
+                varargs = ["*%s" % assigned_names[value]]
+            else:
+                rec, imp, unc = zip(*get_contained_objects_info(value, assigned_names))
+                uncomplete = uncomplete or any(unc)
+                imports.update(union(*imp))
+                varargs = list(rec)
+        elif definition and definition.is_kwarg(arg):
+            if value in assigned_names.keys():
+                kwargs = ["**%s" % assigned_names[value]]
+            else:
+                for karg, kvalue in map_as_kwargs(value):
+                    valuecs = constructor_as_string(kvalue, assigned_names)
+                    uncomplete = uncomplete or valuecs.uncomplete
+                    imports.update(valuecs.imports)
+                    kwargs.append("%s=%s" % (karg, valuecs))
+        else:
+            constructor = constructor_as_string(value, assigned_names)
+            uncomplete = uncomplete or constructor.uncomplete
+            imports.update(constructor.imports)
+            arguments.append("%s=%s" % (arg, constructor))
+    return CallString("%s(%s)" % (object_name, ', '.join(arguments + varargs + kwargs)),
                       uncomplete=uncomplete, imports=imports)
 
+# :: MapObject -> {str: SerializedObject}
+def map_as_kwargs(mapobject):
+    # Keys of kwargs argument must be strings - assertion is checked by
+    # the interpreter on runtime.
+    return sorted([(eval(k.reconstructor), v) for k,v in mapobject.mapping])
+
 # :: SerializedObject | Call | [SerializedObject] -> [SerializedObject]
 def get_contained_objects(obj):
     """Return a list of SerializedObjects this object requires during testing.
@@ -201,6 +283,8 @@ def get_contained_objects(obj):
         return get_those_and_contained_objects(obj.contained_objects)
     elif isinstance(obj, MapObject):
         return get_those_and_contained_objects(flatten(obj.mapping))
+    elif isinstance(obj, BuiltinException):
+        return get_those_and_contained_objects(obj.args)
     elif isinstance(obj, UserObject):
         calls = compact([obj.get_init_call()]) + obj.get_external_calls()
         return get_contained_objects(calls)
@@ -360,8 +444,8 @@ def arguments_as_string(args, always_use_argnames=False):
 def objcall2testname(object_name, args, output):
     """Generate a test method name that describes given object call.
 
-    >>> from test.helper import EmptyProjectExecution
-    >>> serialize = EmptyProjectExecution().serialize
+    >>> from test.helper import make_fresh_serialize
+    >>> serialize = make_fresh_serialize()
 
     >>> objcall2testname('do_this', {}, serialize(True))
     'test_do_this_returns_true'
@@ -592,7 +676,36 @@ class TestGenerator(object):
             module = project.find_module_by_full_path(modname)
             self._add_tests_for_module(module, project, force)
 
-    def create_test_class(self, class_name, method_descriptions):
+    def comment_assertion(self, comment):
+        return comment
+
+    def equal_stub_assertion(self, expected, actual):
+        return "# %s" % self.equal_assertion(expected, actual)
+
+    def raises_stub_assertion(self, exception, code):
+        return "# %s" % self.raises_assertion(exception, code)
+
+    def _add_tests_for_module(self, module, project, force):
+        log.info("Generating tests for module %s." % module.subpath)
+        for test_case in self._generate_test_cases(module):
+            add_test_case_to_project(project, test_case, self.main_snippet, force)
+
+    def _generate_test_cases(self, module):
+        for object in module.testable_objects:
+            test_case = self._generate_test_case(object, module)
+            if test_case:
+                yield test_case
+
+    def _generate_test_case(self, object, module):
+        class_name = name2testname(camelize(object.name))
+        method_descriptions = sorted_test_method_descriptions(self._generate_test_method_descriptions(object, module))
+
+        # Don't generate empty test classes.
+        if method_descriptions:
+            body = self._generate_test_class_code(class_name, method_descriptions)
+            return self._generate_test_class(class_name, method_descriptions, module, body)
+
+    def _generate_test_class_code(self, class_name, method_descriptions):
         result = "%s\n" % (self.test_class_header(class_name))
         for method_description in method_descriptions:
             result += "    def %s(self):\n" % method_description.name
@@ -607,43 +720,16 @@ class TestGenerator(object):
             result += "\n"
         return result
 
-    def comment_assertion(self, comment):
-        return comment
-
-    def equal_stub_assertion(self, expected, actual):
-        return "# %s" % self.equal_assertion(expected, actual)
-
-    def raises_stub_assertion(self, exception, code):
-        return "# %s" % self.raises_assertion(exception, code)
-
-    def _add_tests_for_module(self, module, project, force):
-        log.info("Generating tests for module %s." % module.subpath)
-        for test_case in self._generate_test_cases(module):
-            add_test_case_to_project(project, test_case, force)
-
-    def _generate_test_cases(self, module):
-        for object in module.testable_objects:
-            test_case = self._generate_test_case(object, module)
-            if test_case:
-                yield test_case
-
-    def _generate_test_case(self, object, module):
-        class_name = name2testname(camelize(object.name))
-        method_descriptions = sorted_test_method_descriptions(self._generate_test_method_descriptions(object, module))
-
-        # Don't generate empty test classes.
-        if method_descriptions:
-            test_body = self.create_test_class(class_name, method_descriptions)
-            test_code = parse_fragment(test_body)
-            def methoddesc2testmethod(method_description):
-                name = method_description.name
-                return TestMethod(name=name, code=find_method_code(test_code, name))
-            return TestClass(name=class_name,
-                             code=test_code,
-                             test_cases=map(methoddesc2testmethod, method_descriptions),
-                             imports=self.imports,
-                             main_snippet=self.main_snippet,
-                             associated_modules=[module])
+    def _generate_test_class(self, class_name, method_descriptions, module, body):
+        code = parse_fragment(body)
+        def methoddesc2testmethod(method_description):
+            name = method_description.name
+            return TestMethod(name=name, code=find_method_code(code, name))
+        return TestClass(name=class_name,
+                         code=code,
+                         test_cases=map(methoddesc2testmethod, method_descriptions),
+                         imports=self.imports,
+                         associated_modules=[module])
 
     def _generate_test_method_descriptions(self, object, module):
         if isinstance(object, Function):
@@ -780,19 +866,13 @@ class TestGenerator(object):
         Generated assertion will be a stub if input of a call cannot be
         constructed or if stub argument is True.
         """
-        callstring = decorate_call(call, call_as_string(name, call.input, assigned_names))
+        callstring = decorate_call(call, call_as_string(name, call.input,
+            call.definition, assigned_names))
 
         self.ensure_imports(callstring.imports)
 
         if call.raised_exception():
-            if callstring.uncomplete or stub:
-                assertion_type = 'raises_stub'
-            else:
-                assertion_type = 'raises'
-            self.ensure_import(call.exception.type_import)
-            return (assertion_type,
-                    call.exception.type_name,
-                    in_lambda(callstring))
+            return self._exception_assertion(call.exception, callstring, stub)
         else:
             if callstring.uncomplete or stub:
                 assertion_type = 'equal_stub'
@@ -813,6 +893,31 @@ class TestGenerator(object):
                 self.ensure_import('types')
                 return (assertion_type, output_type, callstring_type)
 
+    def _exception_assertion(self, exception, callstring, stub):
+        if is_serialized_string(exception):
+            # We generate assertion stub because assertRaises handles string
+            # exceptions by identity, not value. This is also the default
+            # behavior of CPython's except clause, see:
+            #
+            # <http://docs.python.org/reference/executionmodel.html#exceptions>
+            #     Exceptions can also be identified by strings, in which case
+            #     the except clause is selected by object identity.
+            #
+            # TODO: It is possible to write assertRaisesString, which compares
+            # string exceptions by value. We don't use this solution, because
+            # currently in Pythoscope there is no facility for attaching and
+            # using test helpers.
+            return ('raises_stub',
+                    todo_value(exception.reconstructor),
+                    in_lambda(callstring))
+
+        if callstring.uncomplete or stub:
+            assertion_type = 'raises_stub'
+        else:
+            assertion_type = 'raises'
+        self.ensure_import(exception.type_import)
+        return (assertion_type, exception.type_name, in_lambda(callstring))
+
 class UnittestTestGenerator(TestGenerator):
     main_snippet = parse_fragment("if __name__ == '__main__':\n    unittest.main()\n")
 
