@@ -5,17 +5,21 @@ import re
 import time
 import types
 
-from pythoscope.astvisitor import EmptyCode, Newline, create_import, \
-    find_last_leaf, get_starting_whitespace, is_node_of_type, regenerate, \
-    remove_trailing_whitespace
+from pythoscope.astvisitor import find_last_leaf, get_starting_whitespace, \
+    is_node_of_type, remove_trailing_whitespace
+from pythoscope.astbuilder import EmptyCode, Newline, create_import, \
+    insert_after, insert_before, regenerate
 from pythoscope.logger import log
-from pythoscope.serializer import ImmutableObject, MapObject, UnknownObject, \
-    SequenceObject, SerializedObject, is_immutable, is_sequence, is_mapping
-from pythoscope.util import all_of_type, set, module_path_to_name, \
+from pythoscope.serializer import BuiltinException, ImmutableObject, \
+    MapObject, UnknownObject, SequenceObject, SerializedObject, is_immutable, \
+    is_sequence, is_mapping, is_builtin_exception
+from pythoscope.compat import set
+from pythoscope.util import all_of_type, module_path_to_name, \
      write_content_to_file, ensure_directory, DirectoryException, \
      get_last_modification_time, read_file_contents, is_generator_code, \
-     extract_subpath, directories_under, findfirst, contains_active_generator, \
-     map_values, class_name, module_name, string2filename
+     extract_subpath, directories_under, findfirst, generator_has_ended, \
+     map_values, class_name, module_name, starts_with_path, string2filename, \
+     get_generator_from_frame
 
 
 CREATIONAL_METHODS = ['__init__', '__new__']
@@ -322,6 +326,12 @@ class Project(object):
         """
         return extract_subpath(path, self.path)
 
+    def contains_path(self, path):
+        """Returns True if given path is under this project's path and False
+        otherwise.
+        """
+        return starts_with_path(path, self.path)
+
     def iter_test_cases(self):
         for module in self.iter_modules():
             for test_case in module.test_cases:
@@ -468,6 +478,18 @@ class Definition(ObjectInModule):
         self.args = args
         self.is_generator = is_generator
 
+    def is_vararg(self, name):
+        """Return True if given string names a vararg argument for this
+        definition.
+        """
+        return ("*%s" % name) in self.args
+
+    def is_kwarg(self, name):
+        """Return True if given string names a kwarg argument for this
+        definition.
+        """
+        return ("**%s" % name) in self.args
+
 class Callable(object):
     """Dynamic aspect of a callable object. Tracks all calls made to given
     callable.
@@ -581,7 +603,7 @@ class UserObject(Callable, SerializedObject):
 
     # Defined lazily to ease testing - classes may be assigned to modules after
     # creation of UserObject, or never at all.
-    module_name = property(lambda self: self.klass.module.locator)
+    module_name = property(lambda self: self.klass.module.locator, lambda s,v: None)
 
     def get_init_call(self):
         """Return a call to __init__ or None if it wasn't called.
@@ -768,22 +790,21 @@ class TestClass(ObjectInModule, TestSuite):
     """Testing class, either generated by Pythoscope or hand-writen by the user.
 
     Each test class contains a set of requirements its surrounding must meet,
-    like the list of imports it needs, contents of the "if __name__ == '__main__'"
-    snippet or specific setup and teardown instructions.
+    like the list of imports it needs or specific setup and teardown
+    instructions.
 
     associated_modules is a list of Modules which this test class exercises.
     """
     allowed_test_case_classes = [TestMethod]
 
     def __init__(self, name, code=None, parent=None, test_cases=[],
-                 imports=None, main_snippet=None, associated_modules=None):
+                 imports=None, associated_modules=None):
         ObjectInModule.__init__(self, name, code)
         TestSuite.__init__(self, parent, imports)
 
         if associated_modules is None:
             associated_modules = []
 
-        self.main_snippet = main_snippet
         self.associated_modules = associated_modules
 
         # Code of test cases passed to the constructor is already contained
@@ -881,7 +902,7 @@ class Module(Localizable, TestSuite):
     allowed_test_case_classes = [TestClass]
 
     def __init__(self, project, subpath, code=None, objects=None, imports=None,
-                 main_snippet=None, errors=[]):
+                 main_snippet=None, last_import=None, errors=[]):
         if objects is None:
             objects = []
 
@@ -896,14 +917,12 @@ class Module(Localizable, TestSuite):
         project.remember_code_tree(code_tree, self)
 
         self.objects = []
-        self.main_snippet = main_snippet
         self.errors = errors
 
-        self.add_objects(objects)
+        self._store_reference('main_snippet', main_snippet)
+        self._store_reference('last_import', last_import)
 
-    def _get_code(self):
-        return code_of(self)
-    code = property(_get_code)
+        self.add_objects(objects)
 
     def _set_module_for_object(self, obj):
         if isinstance(obj, (Class, Function)):
@@ -929,6 +948,9 @@ class Module(Localizable, TestSuite):
         return all_of_type(self.objects, TestClass)
     test_classes = property(_get_test_classes)
 
+    def _store_reference(self, name, code):
+        CodeTree.of(self).add_object(name, code)
+
     def add_objects(self, objects):
         """Add objects to this module.
 
@@ -963,17 +985,15 @@ class Module(Localizable, TestSuite):
     def add_test_case(self, test_case, append_code=True):
         TestSuite.add_test_case(self, test_case, append_code)
         self.add_object(test_case)
-
         self.ensure_imports(test_case.imports)
-        self._ensure_main_snippet(test_case.main_snippet)
 
     def _remove_test_case(self, test_case):
         TestSuite._remove_test_case(self, test_case)
         self.remove_object(test_case)
 
     # def replace_test_case:
-    #   Using the default definition. We don't remove imports or main_snippet,
-    #   because we may unintentionally break something.
+    #   Using the default definition. We don't remove imports because we may
+    #   unintentionally break something.
 
     def get_content(self):
         return regenerate(code_of(self))
@@ -983,20 +1003,21 @@ class Module(Localizable, TestSuite):
         """
         return [tc for tc in self.test_cases if module in tc.associated_modules]
 
-    def _ensure_main_snippet(self, main_snippet, force=False):
+    def ensure_main_snippet(self, main_snippet, force=False):
         """Make sure the main_snippet is present. Won't overwrite the snippet
         unless force flag is set.
         """
         if not main_snippet:
             return
+        current_main_snippet = code_of(self, 'main_snippet')
 
-        if not self.main_snippet:
-            self.main_snippet = main_snippet
+        if not current_main_snippet:
             code_of(self).append_child(main_snippet)
+            self._store_reference('main_snippet', main_snippet)
             self.mark_as_changed()
         elif force:
-            self.main_snippet.replace(main_snippet)
-            self.main_snippet = main_snippet
+            current_main_snippet.replace(main_snippet)
+            self._store_reference('main_snippet', main_snippet)
             self.mark_as_changed()
 
     def _ensure_import(self, import_desc):
@@ -1009,24 +1030,28 @@ class Module(Localizable, TestSuite):
 
     def _add_import(self, import_desc):
         self.imports.append(import_desc)
-        code_of(self).insert_child(0, create_import(import_desc))
+        self._insert_after_other_imports(create_import(import_desc))
         self.mark_as_changed()
 
+    def _insert_after_other_imports(self, code):
+        last_import = code_of(self, 'last_import')
+        if last_import:
+            insert_after(last_import, code)
+        else:
+            code_of(self).insert_child(0, code)
+        # Just inserted import becomes the last one.
+        self._store_reference('last_import', code)
+
     def _append_test_case_code(self, code):
         # If the main_snippet exists we have to put the new test case
         # before it. If it doesn't we put the test case at the end.
-        if self.main_snippet:
-            self._insert_before_main_snippet(code)
+        main_snippet = code_of(self, 'main_snippet')
+        if main_snippet:
+            insert_before(main_snippet, code)
         else:
             code_of(self).append_child(code)
         self.mark_as_changed()
 
-    def _insert_before_main_snippet(self, code):
-        for i, child in enumerate(code_of(self).children):
-            if child == self.main_snippet:
-                code_of(self).insert_child(i, code)
-                break
-
     def save(self):
         # Don't save the test file unless it has been changed.
         if self.changed:
@@ -1035,7 +1060,7 @@ class Module(Localizable, TestSuite):
             try:
                 self.write(self.get_content())
             except DirectoryException, err:
-                raise ModuleSaveError(self.subpath, err.message)
+                raise ModuleSaveError(self.subpath, err.args[0])
             self.changed = False
 
 def object_id(obj):
@@ -1142,6 +1167,8 @@ class Execution(object):
             return SequenceObject(obj, self.serialize)
         elif is_mapping(obj):
             return MapObject(obj, self.serialize)
+        elif is_builtin_exception(obj):
+            return BuiltinException(obj, self.serialize)
         else:
             return UnknownObject(obj)
 
@@ -1150,11 +1177,11 @@ class Execution(object):
         gobject = GeneratorObject(definition, sargs)
         # Generator objects return None to the tracer when stopped. That
         # extra None we have to filter out manually (see
-        # _fix_generator_objects method). The only way to distinguish
-        # between active and stopped generators is to ask garbage collector
-        # about them. So we temporarily save the generator frame inside the
-        # GeneratorObject, so it can be inspected later.
-        gobject._frame = frame
+        # _fix_generator_objects method). We distinguish between active
+        # and stopped generators using the generator_has_ended() function.
+        # It needs the generator object itself, so we save it for later
+        # inspection inside the GeneratorObject.
+        gobject._generator = get_generator_from_frame(frame)
         return gobject
 
     # :: (type, Definition, Callable, args, code, frame) -> Call
@@ -1188,10 +1215,12 @@ class Execution(object):
 
     # :: (str, dict, code, frame) -> FunctionCall | None
     def create_function_call(self, name, args, code, frame):
-        modulename = self.project._extract_subpath(code.co_filename)
-        function = self.project.find_object(Function, name, modulename)
-        if function:
-            return self.create_call(FunctionCall, function, function, args, code, frame)
+        if self.project.contains_path(code.co_filename):
+            modulename = self.project._extract_subpath(code.co_filename)
+            function = self.project.find_object(Function, name, modulename)
+            if function:
+                return self.create_call(FunctionCall, function, function,
+                                        args, code, frame)
 
     def _retrieve_or_capture(self, obj, capture_callback):
         """Return existing description of the given object or create and return
@@ -1222,13 +1251,12 @@ class Execution(object):
         just bogus Nones placed on generator stop.
         """
         for gobject in self.iter_captured_generator_objects():
-            if not contains_active_generator(gobject._frame) \
+            if generator_has_ended(gobject._generator) \
                    and gobject.output \
                    and gobject.output[-1] == ImmutableObject(None):
                 gobject.output.pop()
-            # Once we know if the generator is active or not, we can discard
-            # the frame.
-            del gobject._frame
+            # Once we know if the generator is active or not, we can discard it.
+            del gobject._generator
 
 class PointOfEntry(Localizable):
     """Piece of code provided by the user that allows dynamic analysis.
@@ -1339,7 +1367,7 @@ def module_of(obj):
     else:
         raise TypeError("Don't know how to find the module of %r" % obj)
 
-# :: ObjectInModule -> hashable
+# :: ObjectInModule | str -> hashable
 def module_level_id(obj):
     """Take an object and return something that unambiguously identifies
     it in the scope of its module.
@@ -1354,8 +1382,21 @@ def module_level_id(obj):
         return ('TestClass', obj.name)
     elif isinstance(obj, TestMethod):
         return ('TestMethod', (obj.parent.name, obj.name))
+    elif isinstance(obj, str):
+        return obj
     else:
         raise TypeError("Don't know how to generate a module-level id for %r" % obj)
 
-def code_of(obj):
-    return CodeTree.of(obj).get_code_of(obj)
+def code_of(obj, reference=None):
+    """Return an AST for the given object.
+
+    It is "code_of(obj)" instead of "obj.code" mostly for explicitness. Objects
+    have code attribute when they are created, but lose it once they are added
+    to a Module (see docstring for ObjectInModule). Existence of code_of
+    decouples a storage method (including caching) from an interface.
+    """
+    if reference is not None:
+        assert isinstance(obj, Module)
+    else:
+        reference = obj
+    return CodeTree.of(obj).get_code_of(reference)
