@@ -8,66 +8,8 @@ import traceback
 import types
 import warnings
 
-# Portability code.
-try:
-    set = set
-except NameError:
-    from sets import Set as set
+from pythoscope.compat import groupby, set, sorted
 
-try:
-    frozenset = frozenset
-except NameError:
-    from sets import ImmutableSet as frozenset
-
-try:
-    sorted = sorted
-except NameError:
-    def sorted(iterable, cmp=cmp, key=None):
-        if key:
-            cmp = lambda x,y: cmp(key(x), key(y))
-        alist = list(iterable)
-        alist.sort(cmp)
-        return alist
-
-try:
-    all = all
-except NameError:
-    def all(iterable):
-        for element in iterable:
-            if not element:
-                return False
-        return True
-
-try:
-    from itertools import groupby
-except ImportError:
-    # Code taken from http://docs.python.org/lib/itertools-functions.html .
-    class groupby(object):
-        def __init__(self, iterable, key=None):
-            if key is None:
-                key = lambda x: x
-            self.keyfunc = key
-            self.it = iter(iterable)
-            self.tgtkey = self.currkey = self.currvalue = xrange(0)
-        def __iter__(self):
-            return self
-        def next(self):
-            while self.currkey == self.tgtkey:
-                self.currvalue = self.it.next() # Exit on StopIteration
-                self.currkey = self.keyfunc(self.currvalue)
-            self.tgtkey = self.currkey
-            return (self.currkey, self._grouper(self.tgtkey))
-        def _grouper(self, tgtkey):
-            while self.currkey == tgtkey:
-                yield self.currvalue
-                self.currvalue = self.it.next() # Exit on StopIteration
-                self.currkey = self.keyfunc(self.currvalue)
-
-try:
-    from os.path import samefile
-except ImportError:
-    def samefile(file1, file2):
-        return os.path.realpath(file1) == os.path.realpath(file2)
 
 def compact(lst):
     "Remove all occurences of None from the given list."
@@ -254,6 +196,11 @@ def get_last_modification_time(path):
         # File may not exist, in which case it was never modified.
         return 0
 
+def starts_with_path(path, prefix):
+    """Return True if given path starts with given prefix and False otherwise.
+    """
+    return os.path.realpath(path).startswith(os.path.realpath(prefix))
+
 def extract_subpath(path, prefix):
     """Remove prefix from given path to generate subpath, so the following
     correspondence is preserved:
@@ -263,6 +210,7 @@ def extract_subpath(path, prefix):
     in terms of physical path (i.e. not necessarily strict string
     equality).
     """
+    prefix = os.path.realpath(prefix)
     prefix_length = len(prefix)
     if not prefix.endswith(os.path.sep):
         prefix_length += 1
@@ -312,12 +260,42 @@ def key_for_value(dictionary, value):
         if v == value:
             return k
 
-def contains_active_generator(frame):
-    return bool(all_of_type(gc.get_referrers(frame), types.GeneratorType))
+def get_generator_from_frame(frame):
+    generators = all_of_type(gc.get_referrers(frame), types.GeneratorType)
+    if generators:
+        return generators[0]
 
 def is_generator_code(code):
     return code.co_flags & 0x20 != 0
 
+def generator_has_ended(generator):
+    """Return True if the generator has been exhausted and False otherwise.
+
+    >>> generator_has_ended(1)
+    Traceback (most recent call last):
+      ...
+    TypeError: argument is not a generator
+    """
+    if not isinstance(generator, types.GeneratorType):
+        raise TypeError("argument is not a generator")
+    return _generator_has_ended(generator)
+
+try:
+    from _util import _generator_has_ended
+except ImportError:
+    if sys.version_info < (2, 5):
+        # In Python 2.4 and earlier we can't reliably tell if a generator
+        # is active or not without going to the C level. We assume it
+        # has ended, as it will be true most of the time in our use case.
+        def _generator_has_ended(generator):
+            return True
+        generator_has_ended.unreliable = True
+    else:
+        # This is a hack that uses the fact that in Python 2.5 and higher
+        # generator frame is garbage collected once the generator has ended.
+        def _generator_has_ended(generator):
+            return generator.gi_frame is None
+
 def compile_without_warnings(stmt):
     """Compile single interactive statement with Python interpreter warnings
     disabled.
