@@ -1,6 +1,10 @@
+"""Module containing code that has to be executed before any of the tests.
+"""
+
 # Make pythoscope importable directly from the test modules.
 import os, sys
-sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..'))
+pythoscope_path = os.path.join(os.path.dirname(__file__), os.pardir)
+sys.path.insert(0, os.path.abspath(pythoscope_path))
 
 # Make sys.stdout the logger's output stream, so nose capture
 # plugin can get hold of it.
