@@ -1,18 +1,19 @@
-import difflib
 import os
-import re
+import shutil
+import sys
+import tempfile
 import types
+import warnings
 
 from StringIO import StringIO
 
-from fixture import TempIO
-from nose.tools import assert_equal
-
 from pythoscope.generator import add_tests_to_project
 from pythoscope.logger import DEBUG, INFO, get_output, log, set_output
 from pythoscope.store import CodeTreesManager, CodeTreeNotFound, Execution, \
     Function, ModuleNotFound, PointOfEntry, Project
-from pythoscope.util import quoted_block, read_file_contents, set
+from pythoscope.compat import set
+from pythoscope.util import quoted_block, read_file_contents, \
+    write_content_to_file
 
 
 UNPICKABLE_OBJECT = types.ClassType('class', (), {})
@@ -30,64 +31,6 @@ def P(path):
     "Convert given path with slashes to proper format for OS we're running on."
     return os.path.join(*path.split("/"))
 
-def assert_length(collection, expected_length):
-    actual_length = len(collection)
-    assert expected_length == actual_length,\
-           "Expected collection to have %d elements, it had %d instead." %\
-           (expected_length, actual_length)
-
-def assert_contains(haystack, needle):
-    assert needle in haystack,\
-           "Expected\n%s\nto contain %r, but it didn't." % (quoted_block(haystack), needle)
-
-def assert_contains_once(haystack, needle):
-    repeated = len(re.findall(re.escape(needle), haystack))
-    assert repeated == 1, "Expected\n%s\nto contain %r once, but it contained it %d times instead." %\
-           (quoted_block(haystack), needle, repeated)
-
-def assert_doesnt_contain(haystack, needle):
-    assert needle not in haystack,\
-           "Expected\n%s\nto NOT contain %r, but it did." % (quoted_block(haystack), needle)
-
-def assert_single_class(info, name):
-    assert_length(info.classes, 1)
-    assert_equal(name, info.classes[0].name)
-
-def assert_function(function, name, args):
-    assert_equal(name, function.name)
-    assert_equal(args, function.args)
-
-def assert_single_function(info, name, args=None):
-    assert_length(info.functions, 1)
-    assert_equal(name, info.functions[0].name)
-    if args is not None:
-        assert_equal(args, info.functions[0].args)
-
-def assert_equal_sets(collection1, collection2):
-    assert_equal(set(collection1), set(collection2))
-
-def assert_equal_strings(s1, s2):
-    assert_equal(s1, s2, "Strings not equal. Diff:\n\n%s" % ''.join(difflib.ndiff(s1.splitlines(True), s2.splitlines(True))))
-
-def assert_not_raises(exception, callable):
-    try:
-        callable()
-    except exception:
-        assert False, "Exception %s has been raised." % exception
-
-def assert_instance(obj, objtype):
-    assert isinstance(obj, objtype), \
-           "Expected object %r to be of type %r, it was of type %r instead." % \
-           (obj, objtype, type(obj))
-
-def assert_matches(regexp, string, anywhere=False):
-    if anywhere:
-        match = re.search
-    else:
-        match = re.match
-    assert match(regexp, string), \
-        "Expected\n%s\nto match r'%s', but it didn't." % (quoted_block(string), regexp)
-
 class PointOfEntryMock(PointOfEntry):
     def __init__(self, project=None, name="poe", content=""):
         if project is None:
@@ -126,36 +69,35 @@ def EmptyProject():
     project.new_tests_directory = "tests"
     return project
 
-def ProjectInDirectory():
-    project_path = TempIO()
-    project_path.mkdir(".pythoscope")
-    project_path.mkdir(P(".pythoscope/code-trees"))
+def ProjectInDirectory(project_path):
+    putdir(project_path, ".pythoscope")
+    putdir(project_path, P(".pythoscope/code-trees"))
     project = Project(project_path)
-    # Save the TempIO reference, so we can delay its destruction later.
-    project._tmpdir = project_path
     return project
 
+def with_modules(project, paths, create_files=True):
+    for path in paths:
+        project.create_module(os.path.join(project.path, path))
+        if create_files:
+            putfile(project.path, path, "")
+    return project
+Project.with_modules = with_modules
+
+def with_points_of_entry(project, paths):
+    poe_path = putdir(project.path, P(".pythoscope/points-of-entry"))
+    for path in paths:
+        putfile(poe_path, path, "")
+    return project
+Project.with_points_of_entry = with_points_of_entry
+
 def ProjectWithModules(paths, project_type=EmptyProject):
     project = project_type()
     for path in paths:
         project.create_module(os.path.join(project.path, path))
     return project
 
-def ProjectWithRealModules(paths):
-    project = ProjectWithModules(paths, ProjectInDirectory)
-    for path in paths:
-        project.path.putfile(path, "")
-    return project
-
-def ProjectWithPointsOfEntryFiles(paths):
-    project = ProjectInDirectory()
-    project.path.mkdir(P(".pythoscope/points-of-entry"))
-    for path in paths:
-        project.path.putfile(os.path.join(P(".pythoscope/points-of-entry"), path), "")
-    return project
-
-def TestableProject(more_modules=[], project_type=ProjectInDirectory):
-    project = ProjectWithModules(["module.py"] + more_modules, project_type)
+def TestableProject(path, more_modules=[]):
+    project = ProjectWithModules(["module.py"] + more_modules, lambda: ProjectInDirectory(path))
     project["module"].add_object(Function("function"))
     return project
 TestableProject.__test__ = False
@@ -163,6 +105,9 @@ TestableProject.__test__ = False
 def EmptyProjectExecution():
     return Execution(EmptyProject())
 
+def make_fresh_serialize():
+    return EmptyProjectExecution().serialize
+
 def get_test_module_contents(project):
     """Get contents of the first test module of a project.
     """
@@ -185,21 +130,78 @@ def get_test_cases(project):
     return list(project.iter_test_cases())
 get_test_cases.__test__ = False
 
+def last_exception_as_string():
+    exc_type, exc_value = sys.exc_info()[:2]
+    # Special case for string exceptions.
+    if isinstance(exc_type, str):
+        return exc_type
+    else:
+        return repr(exc_value)
+
+###############################################################################
+# Temporary directories/files helpers
+
+def tmpdir():
+    return tempfile.mkdtemp(prefix="pythoscope-")
+
+def mkdirs(path):
+    try:
+        os.makedirs(path)
+    except OSError, err:
+        # os.makedirs raises OSError(17, 'File exists') when last part of
+        # the path exists and we don't care.
+        if err.errno != 17:
+            raise
+
+def putfile(directory, filename, contents):
+    filepath = os.path.join(directory, filename)
+    mkdirs(os.path.dirname(filepath))
+    write_content_to_file(contents, filepath)
+    return filepath
+
+def putdir(directory, dirname):
+    dirpath = os.path.join(directory, dirname)
+    mkdirs(dirpath)
+    return dirpath
+
+rmtree = shutil.rmtree
+
 ###############################################################################
 # Test superclasses
 #   Subclass one of those to get a desired test fixture.
 
-class CustomSeparator:
+class Test(object):
+    """Common ancestor for all test classes to make super on setUp and tearDown
+    work properly. We can't use unittest.TestCase, because it doesn't allow
+    nose generator methods.
+    """
+    def setUp(self):
+        pass
+    def tearDown(self):
+        pass
+
+class TempDirectory(Test):
+    def setUp(self):
+        self.tmpdir = tmpdir()
+        super(TempDirectory, self).setUp()
+
+    def tearDown(self):
+        rmtree(self.tmpdir)
+        super(TempDirectory, self).tearDown()
+
+class CustomSeparator(Test):
     """Subclass CustomSeparator to test your code with alternative os.path.sep.
     """
     def setUp(self):
         self.old_sep = os.path.sep
         os.path.sep = '#'
+        super(CustomSeparator, self).setUp()
 
     def tearDown(self):
         os.path.sep = self.old_sep
+        super(CustomSeparator, self).tearDown()
 
-class CapturedLogger:
+class CapturedLogger(Test):
     """Capture all log output and make it available to test via
     _get_log_output() method.
     """
@@ -211,13 +213,24 @@ class CapturedLogger:
         self.captured = StringIO()
         set_output(self.captured)
         log.level = self.log_level
+        super(CapturedLogger, self).setUp()
 
     def tearDown(self):
         set_output(self._old_output)
         log.level = self._old_level
+        super(CapturedLogger, self).tearDown()
 
     def _get_log_output(self):
         return self.captured.getvalue()
 
 class CapturedDebugLogger(CapturedLogger):
     log_level = DEBUG
+
+class IgnoredWarnings(Test):
+    def setUp(self):
+        warnings.filterwarnings('ignore')
+        super(IgnoredWarnings, self).setUp()
+
+    def tearDown(self):
+        warnings.resetwarnings()
+        super(IgnoredWarnings, self).tearDown()
