@@ -1,13 +1,9 @@
-from nose.tools import assert_equal
+from pythoscope.astvisitor import ASTVisitor
+from pythoscope.astbuilder import parse, regenerate
 
-from pythoscope.astvisitor import parse, regenerate, ASTVisitor
+from assertions import *
 
 
-class TestParser:
-    def test_handles_inputs_without_newline(self):
-        tree = parse("42 # answer")
-        assert_equal("42 # answer", regenerate(tree))
-
 class TestASTVisitorImports:
     def test_handles_simple_imports(self):
         code = "import unittest"
@@ -84,7 +80,7 @@ class TestASTVisitorImports:
     def _test_import(self, code, method):
         method_called = [False]
         class TestVisitor(ASTVisitor):
-            def visit_import(self, names, import_from):
+            def visit_import(self, names, import_from, body):
                 method(names, import_from)
                 method_called[0] = True
 
