@@ -1,40 +1,50 @@
 import array
 import os
 import re
-import sets
 import sys
 import time
 import types
 
-from nose.exc import SkipTest
-from nose.tools import assert_equal, assert_not_equal, assert_raises
+from nose import SkipTest
 
-from pythoscope.astvisitor import parse
 from pythoscope.generator import add_tests_to_project, constructor_as_string
-from pythoscope.serializer import ImmutableObject, UnknownObject
-from pythoscope.store import Project, Class, Function, Method, \
-     ModuleNeedsAnalysis, ModuleSaveError, TestClass, TestMethod, \
-     MethodCall, FunctionCall, UserObject, GeneratorObject, CodeTree
-from pythoscope.util import read_file_contents, get_last_modification_time, \
-    sorted
+from pythoscope.inspector.static import inspect_code
+from pythoscope.serializer import ImmutableObject
+from pythoscope.store import Class, Function, Method, ModuleNeedsAnalysis, \
+    ModuleSaveError, TestClass, TestMethod, MethodCall, FunctionCall, \
+    UserObject, GeneratorObject
+from pythoscope.compat import sets, sorted
+from pythoscope.util import read_file_contents, get_last_modification_time
 
+from assertions import *
 from helper import CapturedDebugLogger, CapturedLogger, P, \
-    ProjectInDirectory, ProjectWithModules, TestableProject, assert_contains, \
-    assert_contains_once, assert_doesnt_contain, assert_equal_sets, \
-    assert_length, assert_matches, generate_single_test_module, get_test_cases, \
-    EmptyProjectExecution, assert_equal_strings
+    ProjectInDirectory, EmptyProject, TestableProject, \
+    generate_single_test_module, get_test_cases, \
+    EmptyProjectExecution, putfile, TempDirectory, make_fresh_serialize
 
 # Let nose know that those aren't test functions/classes.
 add_tests_to_project.__test__ = False
 TestClass.__test__ = False
 TestMethod.__test__ = False
 
+def stable_serialize_call_arguments(execution, args):
+    """Work just like execution.serialize_call_arguments, but serialize
+    arguments in lexicographical order, so test outputs are stable.
+
+    This doesn't change semantics of the serialization process (as objects
+    identity will be preserved), yet it makes testing it much easier.
+    """
+    serialized_args = {}
+    for key, value in sorted(args.iteritems()):
+        serialized_args[key] = execution.serialize(value)
+    return serialized_args
+
 def create_method_call(method, args, output, call_type, execution):
-    sargs = execution.serialize_call_arguments(args)
+    sargs = stable_serialize_call_arguments(execution, args)
     if call_type == 'output':
         return MethodCall(method, sargs, output=execution.serialize(output))
     elif call_type == 'exception':
-        return MethodCall(method, sargs, exception=execution.serialize(output()))
+        return MethodCall(method, sargs, exception=execution.serialize(output))
     elif call_type == 'generator':
         return GeneratorObject(method, sargs, map(execution.serialize, output))
 
@@ -63,12 +73,6 @@ def ClassWithInstanceWithoutReconstruction(name):
     # module.py.
     return Class(name), types.ClassType(name, (object,), {})()
 
-def stable_serialize_call_arguments(execution, args):
-    serialized_args = {}
-    for key, value in sorted(args.iteritems()):
-        serialized_args[key] = execution.serialize(value)
-    return serialized_args
-
 def FunctionWithCalls(funcname, calls):
     execution = EmptyProjectExecution()
     function = Function(funcname)
@@ -84,8 +88,8 @@ def FunctionWithExceptions(funcname, calls):
     execution = EmptyProjectExecution()
     function = Function(funcname)
     function.calls = [FunctionCall(function,
-                                   execution.serialize_call_arguments(i),
-                                   exception=execution.serialize(e())) for (i,e) in calls]
+                                   stable_serialize_call_arguments(execution, i),
+                                   exception=execution.serialize(e)) for (i,e) in calls]
     return function
 
 def FunctionWithSingleException(funcname, input, exception):
@@ -95,7 +99,7 @@ def GeneratorWithYields(genname, input, yields):
     execution = EmptyProjectExecution()
     generator = Function(genname, is_generator=True)
     gobject = GeneratorObject(generator,
-                              execution.serialize_call_arguments(input),
+                              stable_serialize_call_arguments(execution, input),
                               map(execution.serialize, yields))
     generator.calls = [gobject]
     return generator
@@ -104,8 +108,8 @@ def GeneratorWithSingleException(genname, input, exception):
     execution = EmptyProjectExecution()
     generator = Function(genname, is_generator=True)
     gobject = GeneratorObject(generator,
-                              execution.serialize_call_arguments(input),
-                              exception=execution.serialize(exception()))
+                              stable_serialize_call_arguments(execution, input),
+                              exception=execution.serialize(exception))
     generator.calls = [gobject]
     return generator
 
@@ -197,62 +201,12 @@ class TestGenerator:
     def test_generates_content_in_right_order(self):
         result = generate_single_test_module(objects=[Function('function')])
 
-        assert re.match(r"import unittest.*?class TestFunction.*?if __name__ == '__main__'", result, re.DOTALL)
+        assert_matches(r"import unittest.*?class TestFunction.*?if __name__ == '__main__'", result)
 
     def test_ignores_test_modules(self):
         result = generate_single_test_module()
         assert_equal("", result)
 
-    def test_uses_existing_destination_directory(self):
-        project = ProjectInDirectory()
-        add_tests_to_project(project, [], 'unittest')
-        # Simply make sure it doesn't raise any exceptions.
-
-    def test_doesnt_generate_test_files_with_no_test_cases(self):
-        project = ProjectWithModules(["module.py"], ProjectInDirectory)
-        test_file = os.path.join(project.path, "test_module.py")
-
-        add_tests_to_project(project, [os.path.join(project.path, "module.py")], 'unittest')
-
-        assert not os.path.exists(test_file)
-
-    def test_doesnt_overwrite_existing_files_which_werent_analyzed(self):
-        TEST_CONTENTS = "# test"
-        project = TestableProject()
-        # File exists, but project does NOT contain corresponding test module.
-        existing_file = os.path.join(project.new_tests_directory, "test_module.py")
-        project.path.putfile(existing_file, TEST_CONTENTS)
-
-        def add_and_save():
-            add_tests_to_project(project, [os.path.join(project.path, "module.py")], 'unittest')
-            project.save()
-
-        assert_raises(ModuleNeedsAnalysis, add_and_save)
-        assert_equal(TEST_CONTENTS, read_file_contents(project._path_for_test("test_module.py")))
-
-    def test_creates_new_test_module_if_no_of_the_existing_match(self):
-        project = TestableProject(["test_other.py"], ProjectInDirectory)
-
-        add_tests_to_project(project, [os.path.join(project.path, "module.py")], 'unittest')
-
-        project_test_cases = get_test_cases(project)
-        assert_length(project_test_cases, 1)
-        assert_length(project["test_other"].test_cases, 0)
-
-    def test_doesnt_overwrite_existing_files_which_were_modified_since_last_analysis(self):
-        TEST_CONTENTS = "# test"
-        project = TestableProject(["test_module.py"])
-        # File exists, and project contains corresponding, but outdated, test module.
-        existing_file = project.path.putfile("test_module.py", TEST_CONTENTS)
-        project["test_module"].created = time.time() - 3600
-
-        def add_and_save():
-            add_tests_to_project(project, [os.path.join(project.path, "module.py")], 'unittest')
-            project.save()
-
-        assert_raises(ModuleNeedsAnalysis, add_and_save)
-        assert_equal(TEST_CONTENTS, read_file_contents(existing_file))
-
     def test_generates_test_case_for_each_function_call_with_numbers(self):
         objects = [FunctionWithSingleCall('square', {'x': 4}, 16)]
 
@@ -382,56 +336,6 @@ class TestGenerator:
         assert_contains(result, "# something = Something(param=<TODO: function>)")
         assert_contains(result, "# self.assertEqual(1, something.method()")
 
-    def test_generates_assert_raises_for_functions_with_exceptions(self):
-        function = FunctionWithSingleException('square', {'x': 'hello'}, TypeError)
-
-        result = generate_single_test_module(objects=[function])
-
-        assert_contains(result, "def test_square_raises_type_error_for_hello(self):")
-        assert_contains(result, "self.assertRaises(TypeError, lambda: square(x='hello'))")
-
-    def test_generates_assert_raises_for_init_methods_with_exceptions(self):
-        klass = ClassWithMethods('Something', [('__init__', [({'x': 123}, ValueError)])], 'exception')
-
-        result = generate_single_test_module(objects=[klass])
-
-        assert_contains(result, "def test_creation_with_123_raises_value_error(self):")
-        assert_contains(result, "self.assertRaises(ValueError, lambda: Something(x=123))")
-
-    def test_generates_assert_raises_stub_for_init_methods_with_exceptions(self):
-        klass = ClassWithMethods('Something', [('__init__', [({'x': lambda: 42}, ValueError)])], 'exception')
-
-        result = generate_single_test_module(objects=[klass])
-
-        assert_contains(result, "def test_creation_with_function_raises_value_error(self):")
-        assert_contains(result, "# self.assertRaises(ValueError, lambda: Something(x=<TODO: function>))")
-
-    def test_generates_assert_raises_for_normal_methods_with_exceptions(self):
-        klass = ClassWithMethods('Something', [('method', [({}, KeyError)])], 'exception')
-
-        result = generate_single_test_module(objects=[klass])
-
-        assert_contains(result, "def test_method_raises_key_error(self):")
-        assert_contains(result, "something = Something()")
-        assert_contains(result, "self.assertRaises(KeyError, lambda: something.method())")
-
-    def test_generates_imports_for_user_defined_exceptions(self):
-        klass = Class("UserDefinedException")
-        function = Function("throw")
-        function.calls = [FunctionCall(function, {}, exception=UserObject(None, klass))]
-
-        result = generate_single_test_module(objects=[function, klass])
-
-        assert_contains(result, "from module import UserDefinedException")
-
-    def test_doesnt_generate_imports_for_builtin_exceptions(self):
-        function = Function("throw")
-        function.calls = [FunctionCall(function, {}, exception=UnknownObject(Exception()))]
-
-        result = generate_single_test_module(objects=[function])
-
-        assert_doesnt_contain(result, "import Exception")
-
     def test_generates_assert_equal_type_for_functions_returning_functions(self):
         objects = [FunctionWithSingleCall('higher', {}, lambda: 42)]
 
@@ -478,14 +382,6 @@ class TestGenerator:
         assert_contains(result, "def test_highest_returns_function_for_function(self):")
         assert_contains(result, "# self.assertEqual(types.FunctionType, type(highest(f=<TODO: function>)))")
 
-    def test_generates_assert_raises_test_stub_for_functions_which_take_functions_as_arguments(self):
-        function = FunctionWithSingleException('high', {'f': lambda: 42}, NotImplementedError)
-
-        result = generate_single_test_module(objects=[function])
-
-        assert_contains(result, "def test_high_raises_not_implemented_error_for_function(self):")
-        assert_contains(result, "# self.assertRaises(NotImplementedError, lambda: high(f=<TODO: function>))")
-
     def test_handles_regular_expression_pattern_objects(self):
         objects = [FunctionWithSingleCall('matches', {'x': re.compile('abcd')}, True)]
 
@@ -546,14 +442,6 @@ class TestGenerator:
         assert_contains(result, "def test_lambdify_yields_function_then_function_for_1(self):")
         assert_contains(result, "self.assertEqual([types.FunctionType, types.FunctionType], map(type, list(islice(lambdify(x=1), 2))))")
 
-    def test_generates_assert_raises_for_generator_functions_with_exceptions(self):
-        objects = [GeneratorWithSingleException('throw', {'string': {}}, TypeError)]
-
-        result = generate_single_test_module(objects=objects)
-
-        assert_contains(result, "def test_throw_raises_type_error_for_dict(self):")
-        assert_contains(result, "self.assertRaises(TypeError, lambda: list(islice(throw(string={}), 1)))")
-
     def test_takes_slice_of_generated_values_list_to_work_around_infinite_generators(self):
         objects = [GeneratorWithYields('nats', {'start': 1}, [1, 2, 3])]
 
@@ -719,10 +607,164 @@ class TestGenerator:
 
         assert_contains(result, "# assert_equal(expected, something(arg1, (narg1, narg2), arg2))")
 
-class TestGeneratorWithTestDirectoryAsFile:
+class TestRaisedExceptions:
+    def test_generates_assert_raises_for_functions_with_exceptions(self):
+        function = FunctionWithSingleException('square', {'x': 'hello'}, TypeError())
+
+        result = generate_single_test_module(objects=[function])
+
+        assert_contains(result, "def test_square_raises_type_error_for_hello(self):")
+        assert_contains(result, "self.assertRaises(TypeError, lambda: square(x='hello'))")
+
+    def test_generates_assert_raises_stub_for_functions_with_string_exceptions(self):
+        function = FunctionWithSingleException('bad_function', {}, "bad error")
+
+        result = generate_single_test_module(objects=[function])
+
+        assert_contains(result, "def test_bad_function_raises_bad_error(self):")
+        assert_contains(result, "# self.assertRaises(<TODO: 'bad error'>, lambda: bad_function())")
+
+    def test_generates_assert_raises_for_init_methods_with_exceptions(self):
+        klass = ClassWithMethods('Something', [('__init__', [({'x': 123}, ValueError())])], 'exception')
+
+        result = generate_single_test_module(objects=[klass])
+
+        assert_contains(result, "def test_creation_with_123_raises_value_error(self):")
+        assert_contains(result, "self.assertRaises(ValueError, lambda: Something(x=123))")
+
+    def test_generates_assert_raises_stub_for_init_methods_with_exceptions(self):
+        klass = ClassWithMethods('Something', [('__init__', [({'x': lambda: 42}, ValueError())])], 'exception')
+
+        result = generate_single_test_module(objects=[klass])
+
+        assert_contains(result, "def test_creation_with_function_raises_value_error(self):")
+        assert_contains(result, "# self.assertRaises(ValueError, lambda: Something(x=<TODO: function>))")
+
+    def test_generates_assert_raises_for_normal_methods_with_exceptions(self):
+        klass = ClassWithMethods('Something', [('method', [({}, KeyError())])], 'exception')
+
+        result = generate_single_test_module(objects=[klass])
+
+        assert_contains(result, "def test_method_raises_key_error(self):")
+        assert_contains(result, "something = Something()")
+        assert_contains(result, "self.assertRaises(KeyError, lambda: something.method())")
+
+    def test_generates_imports_for_user_defined_exceptions(self):
+        klass = Class("UserDefinedException")
+        function = Function("throw")
+        function.calls = [FunctionCall(function, {}, exception=UserObject(None, klass))]
+
+        result = generate_single_test_module(objects=[function, klass])
+
+        assert_contains(result, "from module import UserDefinedException")
+
+    def test_doesnt_generate_imports_for_builtin_exceptions(self):
+        function = FunctionWithSingleException('throw', {}, Exception())
+
+        result = generate_single_test_module(objects=[function])
+
+        assert_doesnt_contain(result, "import Exception")
+
+    def test_generates_assert_raises_test_stub_for_functions_which_take_functions_as_arguments(self):
+        function = FunctionWithSingleException('high', {'f': lambda: 42}, NotImplementedError())
+
+        result = generate_single_test_module(objects=[function])
+
+        assert_contains(result, "def test_high_raises_not_implemented_error_for_function(self):")
+        assert_contains(result, "# self.assertRaises(NotImplementedError, lambda: high(f=<TODO: function>))")
+
+    def test_generates_assert_raises_for_generator_functions_with_exceptions(self):
+        objects = [GeneratorWithSingleException('throw', {'string': {}}, TypeError())]
+
+        result = generate_single_test_module(objects=objects)
+
+        assert_contains(result, "def test_throw_raises_type_error_for_dict(self):")
+        assert_contains(result, "self.assertRaises(TypeError, lambda: list(islice(throw(string={}), 1)))")
+
+class TestExceptionsPassedAsValues:
+    def test_generates_assert_equal_for_exception_returned_as_value(self):
+        objects = [FunctionWithSingleCall('error_factory', {}, MemoryError(0, "OOM!"))]
+
+        result = generate_single_test_module(objects=objects)
+
+        assert_contains(result, "def test_error_factory_returns_memory_error(self):")
+        assert_contains(result, "self.assertEqual(MemoryError(0, 'OOM!'), error_factory())")
+
+    def test_handles_environment_error_with_two_arguments(self):
+        objects = [FunctionWithSingleCall('arg_list_too_long',
+                                          {}, EnvironmentError(7, 'Arg list too long'))]
+
+        result = generate_single_test_module(objects=objects)
+
+        assert_contains(result, "def test_arg_list_too_long_returns_environment_error(self):")
+        assert_contains(result, "self.assertEqual(EnvironmentError(7, 'Arg list too long'), arg_list_too_long())")
+
+    def test_handles_environment_error_with_three_arguments(self):
+        objects = [FunctionWithSingleCall('bad_address',
+                                          {}, EnvironmentError(14, 'Bad address', 'module.py'))]
+
+        result = generate_single_test_module(objects=objects)
+
+        assert_contains(result, "def test_bad_address_returns_environment_error(self):")
+        assert_contains(result, "self.assertEqual(EnvironmentError(14, 'Bad address', 'module.py'), bad_address())")
+
+class TestGeneratorOnTheFilesystem(TempDirectory):
+    def test_uses_existing_destination_directory(self):
+        project = ProjectInDirectory(self.tmpdir)
+        add_tests_to_project(project, [], 'unittest')
+        # Simply make sure it doesn't raise any exceptions.
+
+    def test_doesnt_generate_test_files_with_no_test_cases(self):
+        project = ProjectInDirectory(self.tmpdir).with_modules(["module.py"])
+        test_file = os.path.join(project.path, "test_module.py")
+
+        add_tests_to_project(project, [os.path.join(project.path, "module.py")], 'unittest')
+
+        assert not os.path.exists(test_file)
+
+    def test_doesnt_overwrite_existing_files_which_werent_analyzed(self):
+        TEST_CONTENTS = "# test"
+        project = TestableProject(self.tmpdir)
+        # File exists, but project does NOT contain corresponding test module.
+        existing_file = os.path.join(project.new_tests_directory, "test_module.py")
+        putfile(project.path, existing_file, TEST_CONTENTS)
+
+        def add_and_save():
+            add_tests_to_project(project, [os.path.join(project.path, "module.py")], 'unittest')
+            project.save()
+
+        assert_raises(ModuleNeedsAnalysis, add_and_save)
+        assert_equal(TEST_CONTENTS, read_file_contents(project._path_for_test("test_module.py")))
+
+    def test_creates_new_test_module_if_no_of_the_existing_match(self):
+        project = TestableProject(self.tmpdir, ["test_other.py"])
+
+        add_tests_to_project(project, [os.path.join(project.path, "module.py")], 'unittest')
+
+        project_test_cases = get_test_cases(project)
+        assert_length(project_test_cases, 1)
+        assert_length(project["test_other"].test_cases, 0)
+
+    def test_doesnt_overwrite_existing_files_which_were_modified_since_last_analysis(self):
+        TEST_CONTENTS = "# test"
+        project = TestableProject(self.tmpdir, ["test_module.py"])
+        # File exists, and project contains corresponding, but outdated, test module.
+        existing_file = putfile(project.path, "test_module.py", TEST_CONTENTS)
+        project["test_module"].created = time.time() - 3600
+
+        def add_and_save():
+            add_tests_to_project(project, [os.path.join(project.path, "module.py")], 'unittest')
+            project.save()
+
+        assert_raises(ModuleNeedsAnalysis, add_and_save)
+        assert_equal(TEST_CONTENTS, read_file_contents(existing_file))
+
+class TestGeneratorWithTestDirectoryAsFile(TempDirectory):
     def setUp(self):
-        self.project = TestableProject()
-        self.project.path.putfile(self.project.new_tests_directory, "its content")
+        super(TestGeneratorWithTestDirectoryAsFile, self).setUp()
+
+        self.project = TestableProject(self.tmpdir)
+        putfile(self.project.path, self.project.new_tests_directory, "its content")
         self.module_path = os.path.join(self.project.path, "module.py")
         def add_and_save():
             add_tests_to_project(self.project, [self.module_path], 'unittest')
@@ -734,13 +776,12 @@ class TestGeneratorWithTestDirectoryAsFile:
 
     def test_doesnt_save_pickle_file_if_module_save_error_is_raised(self):
         mtime = get_last_modification_time(self.project._get_pickle_path())
-        try: self.add_and_save()
-        except ModuleSaveError: pass
+        assert_raises(ModuleSaveError, self.add_and_save)
         assert_equal(mtime, get_last_modification_time(self.project._get_pickle_path()))
 
 class TestGeneratorWithSingleModule:
     def setUp(self):
-        self.project = ProjectWithModules(["module.py", "test_module.py"])
+        self.project = EmptyProject().with_modules(["module.py", "test_module.py"], create_files=False)
         self.project["module"].add_object(Function("function"))
         self.module_path = os.path.join(self.project.path, "module.py")
 
@@ -754,10 +795,8 @@ class TestGeneratorWithSingleModule:
         assert_equal_sets(['unittest', ('nose', 'SkipTest')], self.project["test_module"].imports)
 
     def test_appends_new_test_classes_to_existing_test_files(self):
-        module = self.project["test_module"]
         TEST_CONTENTS = "class TestSomething: pass\n\n"
-        code_tree = CodeTree(parse(TEST_CONTENTS))
-        self.project.remember_code_tree(code_tree, module)
+        module = inspect_code(self.project, "test_module.py", TEST_CONTENTS)
 
         add_tests_to_project(self.project, [self.module_path], 'unittest')
 
@@ -806,7 +845,7 @@ class TestGeneratorWithSingleModule:
 class TestGeneratorMessages(CapturedLogger):
     def test_reports_each_module_it_generates_tests_for(self):
         paths = ["first.py", "another.py", P("one/more.py")]
-        project = ProjectWithModules(paths)
+        project = EmptyProject().with_modules(paths, create_files=False)
 
         add_tests_to_project(project, paths, 'unittest')
 
@@ -825,7 +864,7 @@ class TestGeneratorMessages(CapturedLogger):
 
 class TestGeneratorDebugMessages(CapturedDebugLogger):
     def test_debug_output_includes_packages_and_module_names(self):
-        project = ProjectWithModules(["module.py"])
+        project = EmptyProject().with_modules(["module.py"], create_files=False)
         project["module"].add_object(Function('some_function'))
 
         add_tests_to_project(project, ["module"], 'unittest')
@@ -837,7 +876,7 @@ class TestGeneratorDebugMessages(CapturedDebugLogger):
 
 class TestConstructorAsString:
     def setUp(self):
-        self.serialize = EmptyProjectExecution().serialize
+        self.serialize = make_fresh_serialize()
 
     def test_reconstructs_set_from_sets_module(self):
         call_string = constructor_as_string(self.serialize(sets.Set([1, 2, 3])))
