@@ -1,10 +1,8 @@
-from nose.tools import assert_equal
-
-from helper import assert_equal_strings, assert_matches, CapturedLogger, \
-    CapturedDebugLogger, P
-
 from pythoscope.logger import log, path2modname
 
+from assertions import *
+from helper import CapturedLogger, CapturedDebugLogger, P
+
 
 class TestLogger(CapturedLogger):
     def test_info_message_in_normal_mode(self):
